@@ -5,6 +5,7 @@ import signal
 import json
 import platform
 import os
+import sys
 import time
 import traceback
 import uuid
@@ -308,9 +309,17 @@ async def train_model_cli(node: Node, model_name, dataloader, batch_size, iters,
   await hold_outstanding(node)
 
 async def check_exo_home():
-  home, has_read, has_write = await ensure_exo_home(), await has_exo_home_read_access(), await has_exo_home_write_access()
-  if DEBUG >= 1: print(f"exo home directory: {home}")
-  print(f"{has_read=}, {has_write=}")
+  try:
+    home = await ensure_exo_home()
+    has_read = await has_exo_home_read_access()
+    has_write = await has_exo_home_write_access()
+    if DEBUG >= 1: print(f"exo home directory: {home}")
+    print(f"{has_read=}, {has_write=}")
+  except Exception as e:
+    print(f"Error in check_exo_home: {e}")
+    import traceback
+    traceback.print_exc()
+    raise
   if not has_read or not has_write:
     print(f"""
           WARNING: Limited permissions for exo home directory: {home}.
@@ -323,7 +332,10 @@ async def main():
   loop = asyncio.get_running_loop()
 
   try: await check_exo_home()
-  except Exception as e: print(f"Error checking exo home directory: {e}")
+  except Exception as e: 
+    print(f"Error checking exo home directory: {e}")
+    import traceback
+    traceback.print_exc()
 
   if not args.models_seed_dir is None:
     try:
@@ -403,6 +415,8 @@ def run():
             pass  # Handled by signal handler
         except Exception as e:
             print(f"Error: {e}")
+            import traceback
+            traceback.print_exc()
         finally:
             if loop and not loop.is_closed(): 
                 try:
